@@ -2,9 +2,9 @@
 
 # Build zipfiles for Pango on Win32: separate runtime and developer packages
 
-ZIP=/tmp/pango-1.10.0-`date +%Y%m%d`.zip
-DEVZIP=/tmp/pango-dev-1.10.0-`date +%Y%m%d`.zip
-cd /opt/head
+ZIP=/tmp/pango-1.13.1.zip
+DEVZIP=/tmp/pango-dev-1.13.1.zip
+cd /home/behdad/.local
 
 rm $ZIP
 zip $ZIP -@ <<EOF
@@ -17,7 +17,7 @@ bin/libpangowin32-1.0-0.dll
 bin/libpangocairo-1.0-0.dll
 EOF
 
-zip $ZIP lib/pango/1.4.0/modules/*.dll
+zip $ZIP lib/pango/1.5.0/modules/*.dll
 
 rm $DEVZIP
 zip -r $DEVZIP -@ <<EOF
