@@ -19,6 +19,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include <config.h>
 #include <stdlib.h>
 #include <math.h>
 #include <string.h>
@@ -26,7 +27,9 @@
 #include "pango-types.h"
 #include "pango-font.h"
 #include "pango-fontmap.h"
-#include "pango-utils.h"
+#include "pango-impl-utils.h"
+
+static const char bad_font_warning[] = "%s called with bad font, expect ugly output";
 
 struct _PangoFontDescription
 {
@@ -50,34 +53,43 @@ pango_font_description_get_type (void)
   static GType our_type = 0;
   
   if (our_type == 0)
-    our_type = g_boxed_type_register_static ("PangoFontDescription",
+    our_type = g_boxed_type_register_static (I_("PangoFontDescription"),
 					     (GBoxedCopyFunc)pango_font_description_copy,
 					     (GBoxedFreeFunc)pango_font_description_free);
 
   return our_type;
 }
 
+
+static const PangoFontDescription pfd_defaults = {
+  NULL,			/* family_name */
+
+  PANGO_STYLE_NORMAL,	/* style */
+  PANGO_VARIANT_NORMAL,	/* variant */
+  PANGO_WEIGHT_NORMAL,	/* weight */
+  PANGO_STRETCH_NORMAL,	/* stretch */
+
+  0,			/* mask */
+  0,			/* static_family */
+  FALSE,		/* size_is_absolute */
+
+  0,			/* size */
+};
+
 /**
  * pango_font_description_new:
  * 
  * Creates a new font description structure with all fields unset.
  * 
- * Return value: the newly-created #PangoFontDescription. Use
- * pango_font_description_free() to free the result.
+ * Return value: the newly allocated #PangoFontDescription, which
+ *               should be freed using pango_font_description_free().
  **/
 PangoFontDescription *
 pango_font_description_new (void)
 {
-  PangoFontDescription *desc = g_new (PangoFontDescription, 1);
+  PangoFontDescription *desc = g_slice_new (PangoFontDescription);
 
-  desc->mask = 0;
-  desc->family_name = NULL;
-  desc->style = PANGO_STYLE_NORMAL;
-  desc->variant = PANGO_VARIANT_NORMAL;
-  desc->weight = PANGO_WEIGHT_NORMAL;
-  desc->stretch = PANGO_STRETCH_NORMAL;
-  desc->size = 0;
-  desc->size_is_absolute = FALSE;
+  *desc = pfd_defaults;
 
   return desc;
 }
@@ -97,27 +109,11 @@ void
 pango_font_description_set_family (PangoFontDescription *desc,
 				   const char           *family)
 {
-  gchar *old_family = NULL;
-	
   g_return_if_fail (desc != NULL);
 
-  if (desc->family_name && !desc->static_family)
-    old_family = desc->family_name;
-
+  pango_font_description_set_family_static (desc, family ? g_strdup (family) : NULL);
   if (family)
-    {
-      desc->family_name = g_strdup (family);
-      desc->mask |= PANGO_FONT_MASK_FAMILY;
-      desc->static_family = FALSE;
-    }
-  else
-    {
-      desc->family_name = NULL;
-      desc->mask &= ~PANGO_FONT_MASK_FAMILY;
-    }
-
-  if (old_family)
-    g_free (old_family);
+    desc->static_family = FALSE;
 }
 
 /**
@@ -147,12 +143,13 @@ pango_font_description_set_family_static (PangoFontDescription *desc,
   if (family)
     {
       desc->family_name = (char *)family;
-      desc->mask |= PANGO_FONT_MASK_FAMILY;
       desc->static_family = TRUE;
+      desc->mask |= PANGO_FONT_MASK_FAMILY;
     }
   else
     {
-      desc->family_name = NULL;
+      desc->family_name = pfd_defaults.family_name;
+      desc->static_family = pfd_defaults.static_family;
       desc->mask &= ~PANGO_FONT_MASK_FAMILY;
     }
 }
@@ -164,7 +161,9 @@ pango_font_description_set_family_static (PangoFontDescription *desc,
  * Gets the family name field of a font description. See
  * pango_font_description_set_family().
  * 
- * Return value: The family name field. (Will be %NULL if not previously set.)
+ * Return value: the family name field for the font description, or
+ *               %NULL if not previously set.  This has the same life-time
+ *               as the font description itself and should not be freed.
  **/
 G_CONST_RETURN char *
 pango_font_description_get_family (const PangoFontDescription *desc)
@@ -212,7 +211,7 @@ pango_font_description_set_style (PangoFontDescription *desc,
 PangoStyle
 pango_font_description_get_style (const PangoFontDescription *desc)
 {
-  g_return_val_if_fail (desc != NULL, PANGO_STYLE_NORMAL);
+  g_return_val_if_fail (desc != NULL, pfd_defaults.style);
 
   return desc->style;
 }
@@ -249,7 +248,7 @@ pango_font_description_set_variant (PangoFontDescription *desc,
 PangoVariant
 pango_font_description_get_variant (const PangoFontDescription *desc)
 {
-  g_return_val_if_fail (desc != NULL, PANGO_VARIANT_NORMAL);
+  g_return_val_if_fail (desc != NULL, pfd_defaults.variant);
   
   return desc->variant;
 }
@@ -288,7 +287,7 @@ pango_font_description_set_weight (PangoFontDescription *desc,
 PangoWeight
 pango_font_description_get_weight (const PangoFontDescription *desc)
 {
-  g_return_val_if_fail (desc != NULL, PANGO_WEIGHT_NORMAL);
+  g_return_val_if_fail (desc != NULL, pfd_defaults.weight);
   
   return desc->weight;
 }
@@ -325,7 +324,7 @@ pango_font_description_set_stretch (PangoFontDescription *desc,
 PangoStretch
 pango_font_description_get_stretch (const PangoFontDescription *desc)
 {
-  g_return_val_if_fail (desc != NULL, PANGO_STRETCH_NORMAL);
+  g_return_val_if_fail (desc != NULL, pfd_defaults.stretch);
 
   return desc->stretch;
 }
@@ -349,6 +348,7 @@ pango_font_description_set_size (PangoFontDescription *desc,
 				 gint                  size)
 {
   g_return_if_fail (desc != NULL);
+  g_return_if_fail (size >= 0);
   
   desc->size = size;
   desc->size_is_absolute = FALSE;
@@ -365,13 +365,14 @@ pango_font_description_set_size (PangoFontDescription *desc,
  * Return value: the size field for the font description in points or device units.
  *   You must call pango_font_description_get_size_is_absolute()
  *   to find out which is the case. Returns 0 if the size field has not
- *   previously been set.  pango_font_description_get_set_fields() to
+ *   previously been set or it has been set to 0 explicitly.
+ *   Use pango_font_description_get_set_fields() to
  *   find out if the field was explicitely set or not.
  **/
 gint
 pango_font_description_get_size (const PangoFontDescription *desc)
 {
-  g_return_val_if_fail (desc != NULL, 0);
+  g_return_val_if_fail (desc != NULL, pfd_defaults.size);
 
   return desc->size;
 }
@@ -379,7 +380,7 @@ pango_font_description_get_size (const PangoFontDescription *desc)
 /**
  * pango_font_description_set_absolute_size:
  * @desc: a #PangoFontDescription
- * @size: the new size, in pango units. There are #PANGO_SCALE Pango units in one
+ * @size: the new size, in Pango units. There are %PANGO_SCALE Pango units in one
  *   device unit. For an output backend where a device unit is a pixel, a @size
  *   value of 10 * PANGO_SCALE gives a 10 pixel font.
  * 
@@ -393,6 +394,7 @@ pango_font_description_set_absolute_size (PangoFontDescription *desc,
 					  double                size)
 {
   g_return_if_fail (desc != NULL);
+  g_return_if_fail (size >= 0);
   
   desc->size = size;
   desc->size_is_absolute = TRUE;
@@ -406,7 +408,7 @@ pango_font_description_set_absolute_size (PangoFontDescription *desc,
  * Determines whether the size of the font is in points or device units.
  * See pango_font_description_set_size() and pango_font_description_set_absolute_size().
  * 
- * Return value: whether the the size for the font description is in
+ * Return value: whether the size for the font description is in
  *   points or device units.  Use pango_font_description_get_set_fields() to
  *   find out if the size field of the font description was explicitely set or not.
  *
@@ -415,7 +417,7 @@ pango_font_description_set_absolute_size (PangoFontDescription *desc,
 gboolean
 pango_font_description_get_size_is_absolute (const PangoFontDescription *desc)
 {
-  g_return_val_if_fail (desc != NULL, FALSE);
+  g_return_val_if_fail (desc != NULL, pfd_defaults.size_is_absolute);
 
   return desc->size_is_absolute;
 }
@@ -432,7 +434,7 @@ pango_font_description_get_size_is_absolute (const PangoFontDescription *desc)
 PangoFontMask
 pango_font_description_get_set_fields (const PangoFontDescription *desc)
 {
-  g_return_val_if_fail (desc != NULL, 0);
+  g_return_val_if_fail (desc != NULL, pfd_defaults.mask);
 
   return desc->mask;
 }
@@ -442,19 +444,22 @@ pango_font_description_get_set_fields (const PangoFontDescription *desc)
  * @desc: a #PangoFontDescription
  * @to_unset: bitmask of fields in the @desc to unset.
  * 
- * Unsets some of the fields in a #PangoFontDescription. Note that
- * this merely marks the fields cleared, it does not clear the
- * settings for those fields, to clear a family name set with
- * pango_font_description_set_family_static() so that it won't
- * be returned by subsequent calls to pango_font_description_get_family(),
- * you must actually call pango_font_description_set_family (desc, NULL);
+ * Unsets some of the fields in a #PangoFontDescription.  The unset
+ * fields will get back to their default values.
  **/
 void
 pango_font_description_unset_fields (PangoFontDescription *desc,
 				     PangoFontMask         to_unset)
 {
+  PangoFontDescription unset_desc;
+  
   g_return_if_fail (desc != NULL);
 
+  unset_desc = pfd_defaults;
+  unset_desc.mask = to_unset;
+
+  pango_font_description_merge_static (desc, &unset_desc, TRUE);
+
   desc->mask &= ~to_unset;
 }
 
@@ -476,32 +481,15 @@ pango_font_description_merge (PangoFontDescription       *desc,
 			      const PangoFontDescription *desc_to_merge,
 			      gboolean                    replace_existing)
 {
-  PangoFontMask new_mask;
-  
-  g_return_if_fail (desc != NULL);
+  gboolean family_merged = desc_to_merge->family_name && (replace_existing || !desc->family_name);
 
-  if (replace_existing)
-    new_mask = desc_to_merge->mask;
-  else
-    new_mask = desc_to_merge->mask & ~desc->mask;
+  pango_font_description_merge_static (desc, desc_to_merge, replace_existing);
 
-  if (new_mask & PANGO_FONT_MASK_FAMILY)
-    pango_font_description_set_family (desc, desc_to_merge->family_name);
-  if (new_mask & PANGO_FONT_MASK_STYLE)
-    desc->style = desc_to_merge->style;
-  if (new_mask & PANGO_FONT_MASK_VARIANT)
-    desc->variant = desc_to_merge->variant;
-  if (new_mask & PANGO_FONT_MASK_WEIGHT)
-    desc->weight = desc_to_merge->weight;
-  if (new_mask & PANGO_FONT_MASK_STRETCH)
-    desc->stretch = desc_to_merge->stretch;
-  if (new_mask & PANGO_FONT_MASK_SIZE)
+  if (family_merged)
     {
-      desc->size = desc_to_merge->size;
-      desc->size_is_absolute = desc_to_merge->size_is_absolute;
+      desc->family_name = g_strdup (desc->family_name);
+      desc->static_family = FALSE;
     }
-
-  desc->mask |= new_mask;
 }
 
 /**
@@ -609,18 +597,21 @@ pango_font_description_better_match (const PangoFontDescription *desc,
  * 
  * Make a copy of a #PangoFontDescription.
  * 
- * Return value: a newly-allocated #PangoFontDescription. This value
- *               must be freed using pango_font_description_free().
+ * Return value: the newly allocated #PangoFontDescription, which should
+ *               be freed with pango_font_description_free().
  **/
 PangoFontDescription *
 pango_font_description_copy  (const PangoFontDescription  *desc)
 {
-  PangoFontDescription *result = g_new (PangoFontDescription, 1);
+  PangoFontDescription *result = g_slice_new (PangoFontDescription);
 
   *result = *desc;
 
-  result->family_name = g_strdup (result->family_name);
-  result->static_family = FALSE;
+  if (result->family_name)
+    {
+      result->family_name = g_strdup (result->family_name);
+      result->static_family = FALSE;
+    }
 
   return result;
 }
@@ -634,16 +625,17 @@ pango_font_description_copy  (const PangoFontDescription  *desc)
  * be used until @desc is modififed or freed. This is meant to be used
  * when the copy is only needed temporarily.
  * 
- * Return value: a newly-allocated #PangoFontDescription. This value
- *               must be freed using pango_font_description_free().
+ * Return value: the newly allocated #PangoFontDescription, which should
+ *               be freed with pango_font_description_free().
  **/
 PangoFontDescription *
 pango_font_description_copy_static (const PangoFontDescription *desc)
 {
-  PangoFontDescription *result = g_new (PangoFontDescription, 1);
+  PangoFontDescription *result = g_slice_new (PangoFontDescription);
 
   *result = *desc;
-  result->static_family = TRUE;
+  if (result->family_name)
+    result->static_family = TRUE;
 
   return result;
 }
@@ -653,12 +645,14 @@ pango_font_description_copy_static (const PangoFontDescription *desc)
  * @desc1: a #PangoFontDescription
  * @desc2: another #PangoFontDescription
  * 
- * Compares two font descriptions for equality.
+ * Compares two font descriptions for equality. Two font descriptions
+ * are considered equal if the fonts they describe are provably identical.
+ * This means that their maskd do not have to match, as long as other fields
+ * are all the same. (Two font descrptions may result in identical fonts
+ * being loaded, but still compare %FALSE.)
  * 
- * Return value: %TRUE if the two font descriptions are proveably
- *               identical. (Two font descriptions may result in
- *               identical fonts being loaded, but still compare
- *                %FALSE.)
+ * Return value: %TRUE if the two font descriptions are identical,
+ * 		 %FALSE otherwise.
  **/
 gboolean
 pango_font_description_equal (const PangoFontDescription  *desc1,
@@ -667,15 +661,14 @@ pango_font_description_equal (const PangoFontDescription  *desc1,
   g_return_val_if_fail (desc1 != NULL, FALSE);
   g_return_val_if_fail (desc2 != NULL, FALSE);
 
-  return (desc1->mask == desc2->mask &&
-	  desc1->style == desc2->style &&
-	  desc1->variant == desc2->variant &&
-	  desc1->weight == desc2->weight &&
-	  desc1->stretch == desc2->stretch &&
-	  desc1->size == desc2->size &&
-	  desc1->size_is_absolute == desc2->size_is_absolute &&
-	  (desc1->family_name == desc2->family_name ||
-	   (desc1->family_name && desc2->family_name && g_ascii_strcasecmp (desc1->family_name, desc2->family_name) == 0)));
+  return desc1->style == desc2->style &&
+	 desc1->variant == desc2->variant &&
+	 desc1->weight == desc2->weight &&
+	 desc1->stretch == desc2->stretch &&
+	 desc1->size == desc2->size &&
+	 desc1->size_is_absolute == desc2->size_is_absolute &&
+	 (desc1->family_name == desc2->family_name ||
+	  (desc1->family_name && desc2->family_name && g_ascii_strcasecmp (desc1->family_name, desc2->family_name) == 0));
 }
 
 #define TOLOWER(c) \
@@ -702,6 +695,7 @@ case_insensitive_hash (const char *key)
  * 
  * Computes a hash of a #PangoFontDescription structure suitable
  * to be used, for example, as an argument to g_hash_table_new().
+ * The hash value is independent of @desc->mask.
  * 
  * Return value: the hash value.
  **/
@@ -710,23 +704,14 @@ pango_font_description_hash (const PangoFontDescription *desc)
 {
   guint hash = 0;
 
-  hash = desc->mask;
-  
-  if (desc->mask & PANGO_FONT_MASK_FAMILY)
+  if (desc->family_name)
     hash = case_insensitive_hash (desc->family_name);
-  if (desc->mask & PANGO_FONT_MASK_SIZE)
-    {
-      hash ^= desc->size;
-      hash ^= desc->size_is_absolute ? 0xc33ca55a : 0;
-    }
-  if (desc->mask & PANGO_FONT_MASK_STYLE)
-    hash ^= desc->style << 16;
-  if (desc->mask & PANGO_FONT_MASK_VARIANT)
-    hash ^= desc->variant << 18;
-  if (desc->mask & PANGO_FONT_MASK_WEIGHT)
-    hash ^= desc->weight << 16;
-  if (desc->mask & PANGO_FONT_MASK_STRETCH)
-    hash ^= desc->stretch << 26;
+  hash ^= desc->size;
+  hash ^= desc->size_is_absolute ? 0xc33ca55a : 0;
+  hash ^= desc->style << 16;
+  hash ^= desc->variant << 18;
+  hash ^= desc->weight << 16;
+  hash ^= desc->stretch << 26;
 
   return hash;
 }
@@ -745,7 +730,7 @@ pango_font_description_free  (PangoFontDescription  *desc)
       if (desc->family_name && !desc->static_family)
 	g_free (desc->family_name);
 
-      g_free (desc);
+      g_slice_free (PangoFontDescription, desc);
     }
 }
 
@@ -773,24 +758,24 @@ pango_font_descriptions_free (PangoFontDescription **descs,
 typedef struct
 {
   int value;
-  const char *str;
+  const char str[16];
 } FieldMap;
 
 static const FieldMap style_map[] = {
-  { PANGO_STYLE_NORMAL, NULL },
+  { PANGO_STYLE_NORMAL, "" },
   { PANGO_STYLE_OBLIQUE, "Oblique" },
   { PANGO_STYLE_ITALIC, "Italic" }
 };
 
 static const FieldMap variant_map[] = {
-  { PANGO_VARIANT_NORMAL, NULL },
+  { PANGO_VARIANT_NORMAL, "" },
   { PANGO_VARIANT_SMALL_CAPS, "Small-Caps" }
 };
 
 static const FieldMap weight_map[] = {
   { PANGO_WEIGHT_ULTRALIGHT, "Ultra-Light" },
   { PANGO_WEIGHT_LIGHT, "Light" },
-  { PANGO_WEIGHT_NORMAL, NULL },
+  { PANGO_WEIGHT_NORMAL, "" },
   { 500, "Medium" },
   { PANGO_WEIGHT_SEMIBOLD, "Semi-Bold" },
   { PANGO_WEIGHT_BOLD, "Bold" },
@@ -803,7 +788,7 @@ static const FieldMap stretch_map[] = {
   { PANGO_STRETCH_EXTRA_CONDENSED, "Extra-Condensed" },
   { PANGO_STRETCH_CONDENSED,       "Condensed" },
   { PANGO_STRETCH_SEMI_CONDENSED,  "Semi-Condensed" },
-  { PANGO_STRETCH_NORMAL,          NULL },
+  { PANGO_STRETCH_NORMAL,          "" },
   { PANGO_STRETCH_SEMI_EXPANDED,   "Semi-Expanded" },
   { PANGO_STRETCH_EXPANDED,        "Expanded" },
   { PANGO_STRETCH_EXTRA_EXPANDED,  "Extra-Expanded" },
@@ -867,7 +852,7 @@ parse_size (const char *word,
   char *end;
   double size = g_ascii_strtod (word, &end);
 
-  if (end - word == wordlen) /* word is a valid float */
+  if ((size_t)(end - word) == wordlen && size >= 0 && size <= 1000000) /* word is a valid float */
     {
       if (pango_size)
 	*pango_size = (int)(size * PANGO_SCALE + 0.5);
@@ -905,7 +890,7 @@ pango_font_description_from_string (const char *str)
 
   g_return_val_if_fail (str != NULL, NULL);
 
-  desc = g_new (PangoFontDescription, 1);
+  desc = g_slice_new (PangoFontDescription);
 
   desc->mask = PANGO_FONT_MASK_STYLE |
                PANGO_FONT_MASK_WEIGHT |
@@ -984,7 +969,7 @@ append_field (GString *str, const FieldMap *map, int n_elements, int val)
     {
       if (map[i].value == val)
 	{
-	  if (map[i].str)
+	  if (map[i].str && map[i].str[0])
 	    {
 	      if (str->len > 0 && str->str[str->len -1] != ' ')
 		g_string_append_c (str, ' ');
@@ -1141,7 +1126,7 @@ pango_font_get_coverage (PangoFont     *font,
  * pango_font_find_shaper:
  * @font: a #PangoFont
  * @language: the language tag
- * @ch: the ISO-10646 character code.
+ * @ch: a Unicode character.
  * 
  * Finds the best matching shaper for a font for a particular
  * language tag and character point.
@@ -1185,7 +1170,30 @@ pango_font_get_glyph_extents  (PangoFont      *font,
 			       PangoRectangle *ink_rect,
 			       PangoRectangle *logical_rect)
 {
-  g_return_if_fail (font != NULL);
+  if (G_UNLIKELY (!PANGO_IS_FONT (font)))
+    {
+      
+      if (!_pango_warning_history.get_glyph_extents)
+        {
+	  _pango_warning_history.get_glyph_extents = TRUE;
+	  g_warning (bad_font_warning, "pango_font_get_glyph_extents");
+	}
+      if (ink_rect)
+        {
+	  ink_rect->x = PANGO_SCALE;
+	  ink_rect->y = - (PANGO_UNKNOWN_GLYPH_HEIGHT - 1) * PANGO_SCALE;
+	  ink_rect->height = (PANGO_UNKNOWN_GLYPH_HEIGHT - 2) * PANGO_SCALE;
+	  ink_rect->width = (PANGO_UNKNOWN_GLYPH_WIDTH - 2) * PANGO_SCALE;
+        }
+      if (logical_rect)
+        {
+	  logical_rect->x = logical_rect->y = 0;
+	  logical_rect->y = - PANGO_UNKNOWN_GLYPH_HEIGHT * PANGO_SCALE;
+	  logical_rect->height = PANGO_UNKNOWN_GLYPH_HEIGHT * PANGO_SCALE;
+	  logical_rect->width = PANGO_UNKNOWN_GLYPH_WIDTH * PANGO_SCALE;
+	}
+      return;
+    }
 
   PANGO_FONT_GET_CLASS (font)->get_glyph_extents (font, glyph, ink_rect, logical_rect);
 }
@@ -1202,23 +1210,64 @@ pango_font_get_glyph_extents  (PangoFont      *font,
  * be provided to indicate that the metrics should be retrieved that
  * correspond to the script(s) used by that language.
  *
- * Returns: a #PangoFontMetrics object. The caller must call pango_font_metrics_unref()
+ * Return value: a #PangoFontMetrics object. The caller must call pango_font_metrics_unref()
  *   when finished using the object.
  **/
 PangoFontMetrics *
 pango_font_get_metrics (PangoFont        *font,
 			PangoLanguage    *language)
 {
+  if (G_UNLIKELY (!PANGO_IS_FONT (font)))
+    {
+      
+      if (!_pango_warning_history.get_metrics)
+        {
+	  _pango_warning_history.get_metrics = TRUE;
+	  g_warning (bad_font_warning, "pango_font_get_metrics");
+	}
+      return pango_font_metrics_new ();
+    }
+
   return PANGO_FONT_GET_CLASS (font)->get_metrics (font, language);
 }
 
+/**
+ * pango_font_get_font_map:
+ * @font: a #PangoFont
+ * 
+ * Gets the font map for which the font was created.
+ *
+ * Return value: the #PangoFontMap for the font
+ *
+ * Since: 1.10
+ **/
+PangoFontMap *
+pango_font_get_font_map (PangoFont *font)
+{
+  if (G_UNLIKELY (!PANGO_IS_FONT (font)))
+    {
+      
+      if (!_pango_warning_history.get_font_map)
+        {
+	  _pango_warning_history.get_font_map = TRUE;
+	  g_warning (bad_font_warning, "pango_font_get_font_map");
+	}
+      return NULL;
+    }
+
+  if (PANGO_FONT_GET_CLASS (font)->get_font_map)
+    return PANGO_FONT_GET_CLASS (font)->get_font_map (font);
+  else
+    return NULL;
+}
+
 GType
 pango_font_metrics_get_type (void)
 {
   static GType our_type = 0;
   
   if (our_type == 0)
-    our_type = g_boxed_type_register_static ("PangoFontMetrics",
+    our_type = g_boxed_type_register_static (I_("PangoFontMetrics"),
 					     (GBoxedCopyFunc)pango_font_metrics_ref,
 					     (GBoxedFreeFunc)pango_font_metrics_unref);
 
@@ -1239,7 +1288,7 @@ pango_font_metrics_get_type (void)
 PangoFontMetrics *
 pango_font_metrics_new (void)
 {
-  PangoFontMetrics *metrics = g_new0 (PangoFontMetrics, 1);
+  PangoFontMetrics *metrics = g_slice_new0 (PangoFontMetrics);
   metrics->ref_count = 1;
 
   return metrics;
@@ -1249,14 +1298,15 @@ pango_font_metrics_new (void)
  * pango_font_metrics_ref:
  * @metrics: a #PangoFontMetrics structure
  * 
- * Increases the reference count of a font metrics structure.
+ * Increase the reference count of a font metrics structure by one.
  * 
  * Return value: @metrics
  **/
 PangoFontMetrics *
 pango_font_metrics_ref (PangoFontMetrics *metrics)
 {
-  g_return_val_if_fail (metrics != NULL, NULL);
+  if (!metrics)
+    return NULL;
   
   metrics->ref_count++;
 
@@ -1267,20 +1317,21 @@ pango_font_metrics_ref (PangoFontMetrics *metrics)
  * pango_font_metrics_unref:
  * @metrics: a #PangoFontMetrics structure
  * 
- * Decreases the reference count of a font metrics structure. If
+ * Decrease the reference count of a font metrics structure by one. If
  * the result is zero, frees the structure and any associated
  * memory.
  **/
 void
 pango_font_metrics_unref (PangoFontMetrics *metrics)
 {
-  g_return_if_fail (metrics != NULL);
+  if (!metrics)
+    return;
   g_return_if_fail (metrics->ref_count > 0 );
   
   metrics->ref_count--;
   
   if (metrics->ref_count == 0)
-    g_free (metrics);
+    g_slice_free (PangoFontMetrics, metrics);
 }
 
 /**
@@ -1293,7 +1344,7 @@ pango_font_metrics_unref (PangoFontMetrics *metrics)
  * actual drawn ink. It is necessary to lay out the text to figure
  * where the ink will be.)
  * 
- * Return value: the ascent, in pango units. (1 point == #PANGO_SCALE pango units.)
+ * Return value: the ascent, in Pango units. (1 point == %PANGO_SCALE Pango units.)
  **/
 int
 pango_font_metrics_get_ascent (PangoFontMetrics *metrics)
@@ -1313,7 +1364,7 @@ pango_font_metrics_get_ascent (PangoFontMetrics *metrics)
  * actual drawn ink. It is necessary to lay out the text to figure
  * where the ink will be.)
  * 
- * Return value: the descent, in pango units. (1 point == #PANGO_SCALE pango units.)
+ * Return value: the descent, in Pango units. (1 point == %PANGO_SCALE Pango units.)
  **/
 int
 pango_font_metrics_get_descent (PangoFontMetrics *metrics)
@@ -1332,7 +1383,7 @@ pango_font_metrics_get_descent (PangoFontMetrics *metrics)
  * determining the initial size for a window. Actual characters in
  * text will be wider and narrower than this.
  * 
- * Return value: the character width, in pango units. (1 point == #PANGO_SCALE pango units.)
+ * Return value: the character width, in Pango units. (1 point == %PANGO_SCALE Pango units.)
  **/
 int
 pango_font_metrics_get_approximate_char_width (PangoFontMetrics *metrics)
@@ -1349,11 +1400,11 @@ pango_font_metrics_get_approximate_char_width (PangoFontMetrics *metrics)
  * Gets the approximate digit width for a font metrics structure.
  * This is merely a representative value useful, for example, for
  * determining the initial size for a window. Actual digits in
- * text can be wider and narrower than this, though this value
+ * text can be wider or narrower than this, though this value
  * is generally somewhat more accurate than the result of
- * pango_font_metrics_get_approximate_digit_width().
+ * pango_font_metrics_get_approximate_char_width() for digits.
  * 
- * Return value: the digit width, in pango units. (1 point == #PANGO_SCALE pango units.)
+ * Return value: the digit width, in Pango units. (1 point == %PANGO_SCALE Pango units.)
  **/
 int
 pango_font_metrics_get_approximate_digit_width (PangoFontMetrics *metrics)
@@ -1515,7 +1566,9 @@ pango_font_family_list_faces (PangoFontFamily  *family,
  * of pango_font_metrics_get_approximate_char_width() may be affected
  * by double-width characters.  
  * 
- * Return value: TRUE if the family is monospace.
+ * Return value: %TRUE if the family is monospace.
+ *
+ * Since: 1.4
  **/
 gboolean 
 pango_font_family_is_monospace (PangoFontFamily  *family)
@@ -1594,7 +1647,7 @@ pango_font_face_get_face_name (PangoFontFace *face)
  * List the available sizes for a font. This is only applicable to bitmap
  * fonts. For scalable fonts, stores %NULL at the location pointed to by
  * @sizes and 0 at the location pointed to by @n_sizes. The sizes returned
- * are in pango units and are sorted in ascending order.
+ * are in Pango units and are sorted in ascending order.
  *
  * Since: 1.4
  **/
@@ -1613,7 +1666,8 @@ pango_font_face_list_sizes (PangoFontFace  *face,
     PANGO_FONT_FACE_GET_CLASS (face)->list_sizes (face, sizes, n_sizes);
   else
     {
-      *sizes = NULL;
+      if (sizes != NULL)
+	*sizes = NULL;
       *n_sizes = 0;
     }
 }
