@@ -72,7 +72,7 @@ cairo_font_map_iface_init (PangoCairoFontMapIface *iface)
 }
 
 G_DEFINE_TYPE_WITH_CODE (PangoCairoFcFontMap, pango_cairo_fc_font_map, PANGO_TYPE_FC_FONT_MAP,
-    { G_IMPLEMENT_INTERFACE (PANGO_TYPE_CAIRO_FONT_MAP, cairo_font_map_iface_init) });
+    { G_IMPLEMENT_INTERFACE (PANGO_TYPE_CAIRO_FONT_MAP, cairo_font_map_iface_init) })
 
 static void
 pango_cairo_fc_font_map_finalize (GObject *object)
@@ -195,7 +195,7 @@ pango_cairo_fc_font_map_init (PangoCairoFcFontMap *cffontmap)
   cffontmap->library = NULL;
   error = FT_Init_FreeType (&cffontmap->library);
   if (error != FT_Err_Ok)
-    g_error ("pango_cairo_font_map_init: Could not initialize freetype");
+    g_critical ("pango_cairo_font_map_init: Could not initialize freetype");
 
   cffontmap->dpi   = 96.0;
 }
